@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use vars qw($VERSION);
 
-$VERSION = '1.1';
+$VERSION = '1.2';
 
 =head1 NAME
 
@@ -29,6 +29,12 @@ loaded dice
     # create a four-sided die with the squares of 1, 2, 3 and 4
     Games::Dice::Advanced->new(sub { int(1+rand(4)) ** 2 });
 
+    # create a non-numeric die
+    Games::Dice::Advanced->new(sub {
+        my @alphas = qw(C D E F G A B);
+        return $alphas[int rand @alphas];
+    });
+
 =head1 METHODS
 
 =over 4
@@ -56,15 +62,15 @@ sub roll {
     if(ref($self) && $self->isa('Games::Dice::Advanced')) {
         # called as object method
         die("roll() called incorrectly") if(@args);
-	    return &{$self};
+        return &{$self};
     } elsif($self eq 'Games::Dice::Advanced') {
         # called as class method
         @args = ('d6') unless(@args);
-	    return sum(map {
-	        (ref($_) && $_->isa('Games::Dice::Advanced')) ?
-	            $_->roll() :
-	            Games::Dice::Advanced->new($_)->roll()
-	    } @args);
+        return _sum(map {
+            (ref($_) && $_->isa('Games::Dice::Advanced')) ?
+                $_->roll() :
+                Games::Dice::Advanced->new($_)->roll()
+        } @args);
     } else {
         die("Out of cucumber error\n");
     }
@@ -132,26 +138,31 @@ sub new {
 
     if(@args == 2) {
         my($recipe, $mul) = @args;
-	    ($recipe, $mul) = ($mul, $recipe) if(ref($mul) || $mul=~ /\D/);
-	    die("Bad arguments to new()") if(ref($mul) || $mul=~ /\D/);
+        ($recipe, $mul) = ($mul, $recipe) if(ref($mul) || $mul=~ /\D/);
+        die("Bad arguments to new()") if(ref($mul) || $mul=~ /\D/);
 
-	    if($recipe !~ /\D/) {                       # constant
-	        # $self = eval("sub { $recipe * $mul }");
-	        $self = sub { $recipe * $mul };
-	    } elsif($recipe =~ /^d(\d+)$/) {            # dINT
+        if($recipe !~ /\D/) {                       # constant
+            $self = sub { $recipe * $mul };
+        } elsif($recipe =~ /^d(\d+)$/) {            # dINT
             # $self = eval("sub { (1 + int(rand($1))) * $mul }");
             my $faces = $1;
             $self = sub { (1 + int(rand($faces))) * $mul };
-	    } elsif($recipe =~ /^(\d+)d(\d+)/) {        # INTdINT
-	        my($repeats, $faces) = ($1, $2);
-	        $self = sub {
-	            $mul * sum(map { 1 + int(rand($faces)) } (1..$repeats))
-	        };
-	    } elsif(ref($recipe) eq 'CODE') {
-	        $self = sub { $mul * &{$recipe} }
-	    } else {
-	        die("$recipe isn't valid");
-    	}
+        } elsif($recipe =~ /^(\d+)d(\d+)/) {        # INTdINT
+            my($repeats, $faces) = ($1, $2);
+            $self = sub {
+                my $random = _sum(map { 1 + int(rand($faces)) } (1..$repeats));
+                $random *= $mul if($mul != 1 && _die_if_not_number($random));
+                return $random
+            };
+        } elsif(ref($recipe) eq 'CODE') {
+            $self = sub {
+                my $random = &{$recipe};
+                $random *= $mul if($mul != 1 && _die_if_not_number($random));
+                return $random;
+            };
+        } else {
+            die("$recipe isn't valid");
+        }
     } else {
         die("new() called incorrectly");
     }
@@ -163,9 +174,14 @@ sub new {
 
 =cut
 
-sub sum { foldl(sub { shift() + shift(); }, @_); }
+sub _die_if_not_number {
+    $_[0] =~ /^-?\d+(\.\d+)?(e\d+)?$/i ||
+        die("Can't multiply a non-numeric value: $_[0]\n");
+}
 
-sub foldl {
+sub _sum { _foldl(sub { shift() + shift(); }, @_); }
+
+sub _foldl {
   my($f, $z, @xs) = @_;
   $z = $f->($z, $_) foreach(@xs);
   return $z;
@@ -178,24 +194,25 @@ randomness are welcome.
 
 Doesn't support dice with fractional or complex numbers of sides :-)
 
+If you find any bugs please report them on Github, preferably with a test case.
+
 =head1 FEEDBACK
 
-I welcome feedback about my code, including constructive criticism.  And,
-while this is free software (both free-as-in-beer and free-as-in-speech) I
-also welcome payment.  In particular, your bug reports will get moved to
-the front of the queue if you buy me something from my wishlist, which can
-be found at L<http://www.cantrell.org.uk/david/shopping-list/wishlist>.
+I welcome feedback about my code, including constructive criticism.
 
-=head1 AUTHOR
+=head1 AUTHOR, COPYRIGHT and LICENCE
 
-David Cantrell E<lt>F<david@cantrell.org.uk>E<gt>
+Copyright 2024 David Cantrell E<lt>F<david@cantrell.org.uk>E<gt>
 
-=head1 COPYRIGHT
+This software is free-as-in-speech software, and may be used,
+distributed, and modified under the terms of either the GNU
+General Public Licence version 2 or the Artistic Licence. It's
+up to you which one you use. The full text of the licences can
+be found in the files GPL2.txt and ARTISTIC.txt, respectively.
 
-Copyright 2003 David Cantrell
+=head1 CONSPIRACY
 
-This module is free-as-in-speech software, and may be used, distributed,
-and modified under the same terms as Perl itself.
+This module is also free-as-in-mason software.
 
 =cut
 
